/*
 * @(#)ServerMain.java
 *
 * Copyright (c) 2007 masahito suzuki, Inc. All Rights Reserved
 */
package org.maachang.sync ;

import java.io.InterruptedIOException;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.ArrayList;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.maachang.util.net.CreateSocket;

/**
 * サーバメイン処理.
 *
 * @version 2007/05/19
 * @author  masahito suzuki
 * @since   MaachangSync 1.00
 */
public class ServerMain {
    
    /**
     * ログオブジェクト.
     */
    private static final Log LOG = LogFactory.getLog( ServerMain.class ) ;
    
    /**
     * アクセプトタイムアウト.
     */
    private static final int ACCEPT_TIMEOUT = 50 ;
    
    /**
     * オープンチェックタイミング.
     */
    private static final int OPEN_CHECK_TIMING = 15 ;
    
    /**
     * パラメータ : 暗号キーワード.
     */
    private static final String PARAM_KEY = "-k" ;
    
    /**
     * パラメータ : ポート番号.
     */
    private static final String PARAM_PORT = "-p" ;
    
    /**
     * パラメータ : バインドアドレス.
     */
    private static final String PARAM_ADDR = "-a" ;
    
    /**
     * メイン処理.
     */
    public static final void main( String[] args )
        throws Exception {
        new ServerMain().execution( args ) ;
    }
    
    /**
     * サーバメイン実行.
     * <BR><BR>
     * サーバメイン実行処理.
     * <BR>
     * @param args プロセス引数を設定します.
     * @exception Exception 例外.
     */
    public void execution( String[] args )
        throws Exception {
        
        // パラメータを取得.
        String keyword = getKeyword( args ) ;
        int port = getPort( args ) ;
        String address = getAddress( args ) ;
        
        ArrayList cl = null ;
        
        // 開始ログ.
        LOG.info( "## サーバ開始 MaachangSync#ServerMain[" +
            " keyword:" + keyword + " port:" + port + " address:" + address + "]" ) ;
        
        try {
            
            // サーバソケットオープン.
            InetAddress addr = ( address == null ) ? null : InetAddress.getByName( address ) ;
            ServerSocket serverSocket = CreateSocket.serverSocket(
                port,addr,MaachangSyncDef.DEFAULT_BACKLOG ) ;
            serverSocket.setSoTimeout( ACCEPT_TIMEOUT ) ;
            addr = null ;
            
            // オープンログ.
            LOG.info( "## addr:" + serverSocket.getInetAddress().getHostAddress() +
                " port:" + serverSocket.getLocalPort() ) ;
            
            cl = new ArrayList() ;
            
            // アクセプト待ち.
            for( int i = 0 ;; i ++ ) {
                
                // アクセプト.
                try {
                    Socket conn = serverSocket.accept() ;
                    if( conn != null ) {
                        LOG.info( "## accept:" + conn.getLocalAddress().getHostAddress() +
                            " port:" + conn.getLocalPort() ) ;
                        ServerAction sa = new ServerAction( conn,keyword ) ;
                        cl.add( sa ) ;
                    }
                } catch( InterruptedIOException ii ) {
                    // アクセプトタイムアウト.
                }
                
                // クローズ確認.
                if( i >= OPEN_CHECK_TIMING ) {
                    i = -1 ;
                    for( int j = cl.size()-1 ; j >= 0 ; j -- ) {
                        ServerAction sa = ( ServerAction )cl.get( j ) ;
                        if( sa == null || sa.isThread() == false ) {
                            cl.remove( j ) ;
                            if( sa != null ) {
                                sa.clear() ;
                                sa = null ;
                            }
                        }
                    }
                }
                
            }
        } catch( Exception e ) {
            LOG.error( "## サーバエラー",e ) ;
        } finally {
            LOG.info( "## サーバ終了....." ) ;
            if( cl != null ) {
                cl.clear() ;
            }
        }
        
    }
    
    /**
     * キーワードを取得.
     */
    private static final String getKeyword( String[] args ) {
        if( args == null ) {
            return MaachangSyncDef.DEFAULT_KEY ;
        }
        int len = args.length ;
        for( int i = 0 ; i < len ; i ++ ) {
            String name = args[ i ] ;
            if( name == null || ( name = name.trim() ).length() <= 0 ) {
                continue ;
            }
            if( name.toLowerCase().startsWith( PARAM_KEY ) == true ) {
                return name.substring( PARAM_KEY.length() ) ;
            }
        }
        return MaachangSyncDef.DEFAULT_KEY ;
    }
    
    /**
     * ポート番号を取得.
     */
    private static final int getPort( String[] args ) {
        if( args == null ) {
            return MaachangSyncDef.DEFAULT_PORT ;
        }
        int len = args.length ;
        for( int i = 0 ; i < len ; i ++ ) {
            String name = args[ i ] ;
            if( name == null || ( name = name.trim() ).length() <= 0 ) {
                continue ;
            }
            if( name.toLowerCase().startsWith( PARAM_PORT ) == true ) {
                String port = name.substring( PARAM_PORT.length() ) ;
                try {
                    int portInt = Integer.parseInt( port ) ;
                    if( portInt >= 0 && portInt < 65535 ) {
                        return portInt ;
                    }
                } catch( Exception e ) {
                }
            }
        }
        return MaachangSyncDef.DEFAULT_PORT ;
    }
    
    /**
     * バインドアドレスを取得.
     */
    private static final String getAddress( String[] args ) {
        if( args == null ) {
            return null ;
        }
        int len = args.length ;
        for( int i = 0 ; i < len ; i ++ ) {
            String name = args[ i ] ;
            if( name == null || ( name = name.trim() ).length() <= 0 ) {
                continue ;
            }
            if( name.toLowerCase().startsWith( PARAM_ADDR ) == true ) {
                return name.substring( PARAM_ADDR.length() ) ;
            }
        }
        return null ;
    }
    
}
