/*
 * @(#)FileUtil.java
 *
 * Copyright (c) 2007 masahito suzuki, Inc. All Rights Reserved
 */
package org.maachang.util ;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import org.maachang.sync.MaachangSyncDef;

/**
 * ファイルユーティリティ.
 *
 * @version 2007/05/19
 * @author  masahito suzuki
 * @since   MaachangSync 1.00
 */
public class FileUtil {
    
    /**
     * OSファイルスペース.
     */
    public static final String FILE_SPACE = System.getProperty( "file.separator" ) ;
    
    /**
     * 対象のディレクトリを生成.
     * <BR><BR>
     * 対象のディレクトリ情報を生成します.
     * <BR>
     * @param dirName 生成対象のディレクトリ名を設定します.
     * @return String 結果のファイル名が返されます.
     * @exception IOException I/O例外.
     */
    public static final String mkdirs( String dirName )
        throws IOException {
        File fp = null ;
        StringBuffer buf = null ;
        String tmp = null ;
        String ret = null ;
        
        try{
            
            if( dirName == null ){
                throw new IllegalArgumentException( "引数は不正です" ) ;
            }
            
            if( isDirExists( dirName ) == true ){
                
                fp = new File( dirName ) ;
                tmp = fp.getCanonicalPath() ;
                buf = new StringBuffer() ;
                
                buf.append( tmp ) ;
                buf.append( FILE_SPACE ) ;
                ret = buf.toString() ;
                
            }
            else{
                
                fp = new File( dirName ) ;
                tmp = fp.getCanonicalPath() ;
                buf = new StringBuffer() ;
                
                buf.append( tmp ) ;
                buf.append( FILE_SPACE ) ;
                ret = buf.toString() ;
                
                if( fp.mkdirs() == false ){
                    throw new IOException( 
                        "ディレクトリ(" + dirName +
                        ")の作成に失敗しました"
                    ) ;
                }
                
            }
            
        }catch( IOException e ){
            throw e ;
        }finally{
            fp = null ;
            buf = null ;
            tmp = null ;
        }
        
        return ret ;
        
    }
    
    /**
     * パス名をフルパス名に設定.
     * <BR><BR>
     * パス名をフルパス名に設定します.
     * <BR>
     * @param path 対象のパス名を設定します.
     * @return String フルパス名が返されます.
     * @exception IOException I/O例外.
     */
    public static final String getFullPath( String path )
    	throws IOException {
        File fp = new File( path ) ;
        return fp.getCanonicalPath() ;
    }
    
    /**
     * 指定パスから、ファイル名を取得.
     * <BR><BR>
     * 指定パスから、ファイル名を取得します.
     * <BR>
     * @param path 対象のパス名を設定します.
     * @return String 対象のファイル名が返されます.
     */
    public static final String getFileName( String path ) {
        int p = path.lastIndexOf( "\\" ) ;
        int p2 = path.lastIndexOf( "/" ) ;
        if( p == -1 ) {
            if( p2 != -1 ) {
                p = p2 ;
            }
        }
        else if( p2 != -1 && p < p2 ) {
            p = p2 ;
        }
        
        if( p == -1 ) {
            return path ;
        }
        return path.substring( p+1 ) ;
    }
    
    /**
     * 指定位置のファイル区切りを削除.
     * <BR><BR>
     * 指定位置のファイル区切りを削除します.
     * <BR>
     * @param mode 対象の指定位置を設定します.<BR>
     *             [true]を設定した場合、先頭のファイル区切りを外します.
     * @param name 対象のファイル名を設定します.
     * @return String 削除された情報が返されます.
     */
    public static final String cutSeparator( boolean mode,String name ) {
        if( mode == true ) {
            name = name.trim() ;
            if( name.startsWith( "/" ) == true || name.startsWith( "\\" ) == true ) {
                return name.substring( 1 ) ;
            }
        }
        else {
            name = name.trim() ;
            if( name.endsWith( "/" ) == true || name.endsWith( "\\" ) == true ) {
                return name.substring( 0,name.length()-1 ) ;
            }
        }
        return name ;
    }
    
    /**
     * 指定ファイルを削除.
     * <BR><BR>
     * @param name 対象のファイル名を設定します.
     */
    public static final void removeFile( String name ) {
        try {
            File file = new File( name ) ;
            file.delete() ;
        } catch( Exception e ) {
        }
    }
    
    /**
     * 指定ファイルを移動.
     * <BR><BR>
     * @param src 移動元のファイル名を設定します.
     * @param dest 移動先のファイル名を設定します.
     */
    public static final void moveFile( String src,String dest ) {
        try {
            File fileSrc = new File( src ) ;
            File fileDest = new File( dest ) ;
            fileSrc.renameTo( fileDest ) ;
        } catch( Exception e ) {
        }
    }
    
    /**
     * 指定ファイル長を取得.
     * <BR><BR>
     * @param name 対象のファイル名を設定します.
     * @return long 対象のファイル長が返されます.
     */
    public static final long getLength( String name ) {
        long ret = -1L ;
        
        try {
            File file = new File( name ) ;
            ret = ( file.exists() == true ) ? 
                file.length() : -1L ;
        } catch( Exception e ) {
            ret = -1L ;
        }
        
        return ret ;
    }
    
    /**
     * 指定ファイル日付を取得.
     * <BR><BR>
     * @param name 対象のファイル名を設定します.
     * @return long 対象のファイル日付が返されます.
     */
    public static final long getLastTime( String name ) {
        long ret = 0L ;
        
        try {
            File file = new File( name ) ;
            ret = ( file.exists() == true ) ? 
                file.lastModified() : 0L ;
        } catch( Exception e ) {
            ret = 0L ;
        }
        
        return ret ;
    }
    
    /**
     * 指定ファイル名が存在するかチェック.
     * <BR><BR>
     * 指定ファイル名が存在するかチェックします.<BR>
     * <BR>
     * @param name ファイル名を指定します.
     * @return boolean ファイル存在フラグが返されます.<BR>
     *                 [true]が返された場合、指定したファイル名は存在します.<BR>
     *                 [false]が返された場合、指定したファイル名は存在しません.
     */
    public static final boolean isFileExists( String name ) {
        boolean ret ;
        
        if( name == null || name.length() <= 0 ){
            return false ;
        }
        
        try{
            
            File file = new File( name ) ;
            ret = ( file.exists() == true ) ?
                (
                    ( file.isDirectory() == false ) ?
                        file.canRead() : false
                ) 
                : false ;
            
        }catch( Exception e ){
            ret = false ;
        }
        
        return ret ;
    }
    
    /**
     * 指定ディレクトリ名が存在するかチェック.
     * <BR><BR>
     * 指定ディレクトリ名が存在するかチェックします.
     * <BR>
     * @param dirName ディレクトリ名を指定します.
     * @return boolean ディレクトリ存在フラグが返されます.<BR>
     *                 [true]が返された場合、指定したディレクトリ名は存在します.<BR>
     *                 [false]が返された場合、指定したディレクトリ名は存在しません.
     */
    public static final boolean isDirExists( String dirName ) {
        boolean ret ;
        File check = null ;
        
        if( dirName == null ){
            return false ;
        }
        
        check = new File( dirName ) ;
        ret = check.isDirectory() ;
        check = null ;
        
        return ret ;
    }
    
    /**
     * 指定ファイル名が読み取り可能かチェック.
     * <BR><BR>
     * 指定ファイル名が読み取り可能かチェックします.<BR>
     * <BR>
     * @param name 読み取り可能チェック対象のディレクトリ／ファイル名を
     *             指定します.
     * @return boolean 読み取り可能か否かが返されます.<BR>
     *                 [true]が返された場合、読み取り可能です.<BR>
     *                 [false]が返された場合、読み取り不可能か存在しません.
     */
    public static final boolean isRead( String name ) {
        boolean ret ;
        
        File file = null ;
        
        if( name == null || name.length() <= 0 ){
            return false ;
        }
        
        try{
            
            file = new File( name ) ;
            ret = file.canRead() ;
            
        }catch( Exception e ){
            ret = false ;
        }finally{
            file = null ;
        }
        
        return ret ;
    }
    
    /**
     * 指定ファイル名が書き込み可能かチェック.
     * <BR><BR>
     * 指定ファイル名が書き込み可能かチェックします.
     * <BR>
     * @param name 書き込み可能チェック対象のディレクトリ／ファイル名を
     *             指定します.
     * @return boolean 書き込み可能か否かが返されます.<BR>
     *                 [true]が返された場合、書き込み可能です.<BR>
     *                 [false]が返された場合、書き込み不可能か存在しません.
     */
    public static final boolean isWrite( String name ) {
        boolean ret ;
        File check = null ;
        
        if( name == null ){
            return false ;
        }
        
        check = new File( name ) ;
        ret = check.canWrite() ;
        check = null ;
        
        return ret ;
    }
    
    /**
     * 指定ディレクトリ内の最終更新日付を取得.
     * <BR><BR>
     * 指定ディレクトリ内の最終更新日付を取得します.
     * <BR>
     * @param name 対象のディレクトリ名を設定します.
     * @return long ディレクトリ内の情報に対する最終更新日が返されます.
     */
    public static final long getDirectoryToLastTime( String name ) {
        if( isDirExists( name ) == false ) {
            return -1L ;
        }
        return lastUpdateByDirectory( name,-1L ) ;
    }
    
    /**
     * 指定ディレクトリ内の指定時間より後の更新日付のファイル名を取得.
     * <BR><BR>
     * 指定ディレクトリ内の指定時間より後の更新日付のファイル名を取得します.
     * <BR>
     * @param out ファイル名を格納するオブジェクトを設定します.
     * @param name 対象のディレクトリ名を設定します.
     * @param targetTime ターゲットの日付を設定します.
     */
    public static final void getDirectoryToTargetNextTime( ArrayList out,String name,long targetTime ) {
        if( isDirExists( name ) == false ) {
            return ;
        }
        targetNextUpdateByDirectory( out,name,targetTime ) ;
    }
    
    /**
     * ディレクトリ内の最終時間を取得.
     */
    private static final long lastUpdateByDirectory( String name,long ret ) {
        
        if( name.endsWith( "\\" ) == false && name.endsWith( "/" ) == false ) {
            name = name + FILE_SPACE ;
        }
        
        File file = new File( name ) ;
        String[] names = file.list() ;
        file = null ;
        int len ;
        if( names != null && ( len = names.length ) > 0 ) {
            for( int i = 0 ; i < len ; i ++ ) {
                String target = name + names[ i ] ;
                names[ i ] = null ;
                File fp = new File( target ) ;
                
                if( fp.isDirectory() == true ) {
                    fp = null ;
                    ret = lastUpdateByDirectory( target,ret ) ;
                }
                else {
                    long now = fp.lastModified() ;
                    fp = null ;
                    if( getFileName( target ).startsWith( MaachangSyncDef.TEMP_FILENAME ) == false ) {
                        if( now > ret ) {
                            ret = now ;
                        }
                    }
                }
            }
        }
        
        return ret ;
    }
    
    /**
     * 指定ディレクトリ以下にある、ターゲット時間より後のファイル日付の名前を取得.
     */
    private static final void targetNextUpdateByDirectory( ArrayList lst,String name,long targetTime ) {
        
        if( name.endsWith( "\\" ) == false && name.endsWith( "/" ) == false ) {
            name = name + FILE_SPACE ;
        }
        
        File file = new File( name ) ;
        String[] names = file.list() ;
        file = null ;
        int len ;
        if( names != null && ( len = names.length ) > 0 ) {
            for( int i = 0 ; i < len ; i ++ ) {
                String target = name + names[ i ] ;
                names[ i ] = null ;
                File fp = new File( target ) ;
                
                if( fp.isDirectory() == true ) {
                    fp = null ;
                    targetNextUpdateByDirectory( lst,target,targetTime ) ;
                }
                else {
                    long now = fp.lastModified() ;
                    fp = null ;
                    if( getFileName( target ).startsWith( MaachangSyncDef.TEMP_FILENAME ) == false ) {
                        if( now > targetTime ) {
                            lst.add( new String( target ) ) ;
                        }
                    }
                }
            }
        }
        
    }
    
}

