/*
 * @(#)CreateSocket.java
 *
 * Copyright (c) 2007 masahito suzuki, Inc. All Rights Reserved
 */
package org.maachang.util.net ;

import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;

/**
 * ソケット生成処理を行います.
 *  
 * @version 2007/05/19
 * @author  masahito suzuki
 * @since   MaachangSync 1.00
 */
public class CreateSocket {
    
    /**
     * バッファサイズ.
     */
    public static final int BUFFER_LENGTH = 512 ;
    
    /**
     * 受信タイムアウト更新カウント.
     */
    public static final int NEW_RECEIVE_TIMEOUT_COUNT = 64 ;
    
    /**
     * ソケット受信タイムアウト.
     */
    public static final int RECV_TIMEOUT = 30 ;
    
    /**
     * 基本バッファサイズ.
     */
    public static final int BASE_BUFFER = 262144 ;
    
    /**
     * デフォルト最大アクセス.
     */
    private static final int DEFAULT_MAX_ACCESS = 50 ;
    
    /**
     * サーバソケット生成処理を行います.
     * <BR><BR>
     * @param bindPort 対象のバインドポートを設定します.
     * @param bindAddress 対象のバインドアドレスを設定します.<BR>
     *                    [null]の場合、デフォルトアドレスを指定します.
     * @param backlog 最大接続数を設定します.<BR>
     *                [0]以下を設定した場合、デフォルト値を設定します.
     * @return ServerSocket サーバソケットが返されます.
     */
    public static final ServerSocket serverSocket( int bindPort,InetAddress bindAddress,int backlog )
        throws Exception {
        
        if( backlog <= 0 ) {
            backlog = DEFAULT_MAX_ACCESS ;
        }
        
        return new ServerSocket( bindPort,backlog,bindAddress ) ;
    }
    
    /**
     * クライアントソケット生成処理を行います.
     * <BR><BR>
     * @param address 接続先アドレスを設定します.
     * @param port 接続先ポート番号を設定します.
     * @return Socket 対象のソケットオブジェクトが返されます.
     * @exception Exception 例外.
     */
    public static final Socket clientSocket( InetAddress address,int port )
        throws Exception {
        
        return new Socket( address,port ) ;
    }
    
    /**
     * TCP/IPを生成.
     * <BR><BR>
     * TCP/IPを生成します.
     * <BR>
     * @param socket 対象のソケットを設定します.
     * @return Tcp 通信オブジェクトが返されます.
     * @exception Exception 例外.
     */
    public static final Tcp createTcp( Socket socket )
        throws Exception {
        
        if( socket == null || socket.isClosed() == true ) {
            throw new IllegalArgumentException( "引数は不正です" ) ;
        }
        
        socket.setSendBufferSize( BASE_BUFFER ) ;
        socket.setReceiveBufferSize( BASE_BUFFER ) ;
        socket.setSoTimeout( RECV_TIMEOUT ) ;
        socket.setSoLinger( true,1 ) ;
        socket.setKeepAlive( true ) ;
        socket.setTcpNoDelay( true ) ;
        socket.setReuseAddress( true ) ;
        
        Tcp tcp = new Tcp( socket ) ;
        return tcp ;
    }
    
}

